@@ -40,33 +40,23 @@
 
 /* Default size for windows */
 
-#define DEFAULT_WINDOW_WIDTH 400
-#define DEFAULT_WINDOW_HEIGHT 300
+#define DEFAULT_WINDOW_WIDTH 200
+#define DEFAULT_WINDOW_HEIGHT 200
 
 #define EOG_VIEWER_CONTROL_IID "OAFIID:GNOME_EOG_Control"
+#define EOG_WINDOW_DND_POPUP_PATH  "/popups/dragndrop"
 
 /* Private part of the Window structure */
 struct _EogWindowPrivate {
-	/* UIImage widget for our contents */
-	GtkWidget *ui;
-
 	/* Our GConf client */
 	GConfClient *client;
 
-	/* ui container */
-	BonoboUIContainer  *ui_container;
-
 	/* control frame */
 	BonoboControlFrame *ctrl_frame;
 
 	/* vbox */
 	GtkWidget           *box;
 
-	/* the control embedded in the container */
-	GtkWidget           *widget;
-
-	Bonobo_PropertyControl property_control;
-
 	/* statusbar */
 	GtkWidget *statusbar;
 
@@ -75,6 +65,9 @@ struct _EogWindowPrivate {
 
 	/* GConf client notify id's */
 	guint sb_policy_notify_id;
+
+	/* list of files recieved from a drag'n'drop action */
+	GList *dnd_files;
 };
 
 static void eog_window_class_init (EogWindowClass *class);
@@ -84,6 +77,8 @@ static gint eog_window_delete (GtkWidget *widget, GdkEventAny *event);
 static gint eog_window_key_press (GtkWidget *widget, GdkEventKey *event);
 static void eog_window_drag_data_received (GtkWidget *widget, GdkDragContext *context, gint x, gint y, 
 				    GtkSelectionData *selection_data, guint info, guint time);
+static void open_dnd_files (EogWindow *window, gboolean need_new_window);
+
 
 static BonoboWindowClass *parent_class;
 
@@ -168,9 +163,12 @@ verb_HelpAbout_cb (BonoboUIComponent *uic, gpointer user_data, const char *cname
 {
 	static GtkWidget *about;
 	static const char *authors[] = {
-		"Federico Mena-Quintero (federico@gnu.org)",
-		"Arik Devens (arik@gnome.org)",
-		"Jens Finke (jens@gnome.org)",
+		"Federico Mena-Quintero",
+		"Martin Baulig",
+		"Arik Devens",
+		"Jens Finke",
+		"Michael Meeks",
+		"Lutz Mueller",
 		NULL
 	};
 
@@ -178,7 +176,7 @@ verb_HelpAbout_cb (BonoboUIComponent *uic, gpointer user_data, const char *cname
 		about = gnome_about_new (
 			_("Eye of Gnome"),
 			VERSION,
-			_("Copyright (C) 2000-2001 The Free Software Foundation"),
+			_("Copyright (C) 2000-2002 The Free Software Foundation"),
 			_("The GNOME image viewing and cataloging program"),
 			authors,
 			NULL, /* char **documentors */
@@ -193,6 +191,26 @@ verb_HelpAbout_cb (BonoboUIComponent *uic, gpointer user_data, const char *cname
 	raise_and_focus (about);
 }
 
+static void
+verb_DnDNewWindow_cb (BonoboUIComponent *uic, gpointer user_data, const char *cname)
+{
+	open_dnd_files (EOG_WINDOW (user_data), TRUE);
+}
+
+static void
+verb_DnDSameWindow_cb (BonoboUIComponent *uic, gpointer user_data, const char *cname)
+{
+	open_dnd_files (EOG_WINDOW (user_data), FALSE);
+}
+
+
+static void
+verb_DnDCancel_cb (BonoboUIComponent *uic, gpointer user_data, const char *cname)
+{
+	gnome_vfs_uri_list_free (EOG_WINDOW (user_data)->priv->dnd_files);
+	EOG_WINDOW (user_data)->priv->dnd_files = NULL;	
+}
+
 static void
 activate_uri_cb (BonoboControlFrame *control_frame, const char *uri, gboolean relative, gpointer data)
 {
@@ -251,13 +269,17 @@ eog_window_destroy (GtkObject *object)
 	g_return_if_fail (object != NULL);
 	g_return_if_fail (EOG_IS_WINDOW (object));
 
+	if (getenv ("DEBUG_EOG"))
+		g_message ("Destroying EogWindow...");
+
 	window = EOG_WINDOW (object);
 	priv = window->priv;
 
-	if (priv->property_control != CORBA_OBJECT_NIL)
-		bonobo_object_release_unref (priv->property_control, NULL);
-	priv->property_control = CORBA_OBJECT_NIL;
-	
+	if (priv->ctrl_frame != NULL) {
+		bonobo_object_unref (BONOBO_OBJECT (priv->ctrl_frame));
+		priv->ctrl_frame = NULL;
+	}
+
 	/* Clean up GConf-related stuff */
 	if (priv->client) {
 		gconf_client_notify_remove (priv->client, priv->sb_policy_notify_id);
@@ -267,7 +289,7 @@ eog_window_destroy (GtkObject *object)
 	}
 
 	if (GTK_OBJECT_CLASS (parent_class)->destroy)
-		(* GTK_OBJECT_CLASS (parent_class)->destroy) (object);
+		GTK_OBJECT_CLASS (parent_class)->destroy (object);
 }
 
 static void
@@ -311,17 +333,19 @@ eog_window_class_init (EogWindowClass *class)
 static void
 sb_policy_changed_cb (GConfClient *client, guint notify_id, GConfEntry *entry, gpointer data)
 {
-/*	EogWindow *window;
-	EogWindowPrivate *priv;
+/*     EogWindow *window;
+       EogWindowPrivate *priv;
 
-	window = EOG_WINDOW (data);
-	priv = window->priv;
+       window = EOG_WINDOW (data);
+       priv = window->priv;
 
-	priv->sb_policy = gconf_value_get_int (entry->value);
+       priv->sb_policy = gconf_value_get_int (entry->value);
 
-	gtk_scroll_frame_set_policy (GTK_SCROLL_FRAME (priv->ui), priv->sb_policy, priv->sb_policy);*/
+       gtk_scroll_frame_set_policy (GTK_SCROLL_FRAME (priv->ui), priv->sb_policy, priv->sb_policy);*/
 }
 
+
+
 /* Object initialization function for windows */
 static void
 eog_window_init (EogWindow *window)
@@ -331,7 +355,6 @@ eog_window_init (EogWindow *window)
 	priv = g_new0 (EogWindowPrivate, 1);
 	window->priv = priv;
 
-	priv->property_control = CORBA_OBJECT_NIL;
 	priv->client = gconf_client_get_default ();
 
 	gconf_client_add_dir (priv->client, "/apps/eog",
@@ -392,20 +415,13 @@ eog_window_has_contents (EogWindow *window)
 {
 	EogWindowPrivate *priv;
 
+	g_return_val_if_fail (EOG_IS_WINDOW (window), FALSE);
+
 	priv = window->priv;
 
-	return (priv->widget != NULL);
+	return (bonobo_control_frame_get_widget (priv->ctrl_frame) != NULL);
 }
 
-static GnomeUIInfo drag_ask_popup_menu [] = {
-	GNOMEUIINFO_ITEM_NONE (N_("Open in new window"), NULL, NULL),
-	GNOMEUIINFO_ITEM_NONE (N_("Open in this window"), NULL, NULL), 
-	GNOMEUIINFO_SEPARATOR,
-	GNOMEUIINFO_ITEM_STOCK (N_("Cancel"), NULL , NULL, GNOME_STOCK_BUTTON_CANCEL),
-	GNOMEUIINFO_END
-};
-
-
 /* Drag_data_received handler for windows */
 static void
 eog_window_drag_data_received (GtkWidget *widget, 
@@ -426,42 +442,46 @@ eog_window_drag_data_received (GtkWidget *widget,
 	if (info != TARGET_URI_LIST)
 		return;
 
+	if (priv->dnd_files != NULL)
+		gnome_vfs_uri_list_free (priv->dnd_files);
+	priv->dnd_files = gnome_vfs_uri_list_parse (selection_data->data);
+
 	if (context->suggested_action == GDK_ACTION_ASK) {
-		GtkWidget *menu = gnome_popup_menu_new (drag_ask_popup_menu);
-		int i = gnome_popup_menu_do_popup_modal (menu, NULL, NULL,
-						     NULL, NULL, NULL);
-		gtk_object_unref (GTK_OBJECT (menu));
-		switch (i) {
-		case 0:
-			need_new_window = TRUE;
-			break;
-		case 1:
-			need_new_window = FALSE;
-			break;
-		default:
-			return;
-			break;
-		}
+		GtkWidget *menu = gtk_menu_new ();
+		
+		bonobo_window_add_popup (BONOBO_WINDOW (window), 
+					 GTK_MENU (menu), 
+					 EOG_WINDOW_DND_POPUP_PATH);
+		gtk_menu_popup (GTK_MENU (menu),
+				NULL,
+				NULL,
+				NULL,
+				NULL,
+				0,
+				GDK_CURRENT_TIME);
 	} else {
-		/* The first image is opened in the same window only if the
+ 		/* The first image is opened in the same window only if the
 		* current window has no image in it.
 		*/
 		need_new_window = eog_window_has_contents (window);		
+		open_dnd_files (window, need_new_window);
 	}
+}
 
-#if 0
-	/* FIXME: This should use GnomeVFS later and it should not strip the
-	 * method prefix.
-	 */
-	filenames = gnome_uri_list_extract_filenames (selection_data->data);
-#endif
+static void
+open_dnd_files (EogWindow *window, gboolean need_new_window)
+{
+	GtkWidget *new_window;
+	GList *l;
+	char *filename;
 
-	for (l = filenames; l; l = l->next) {
-		GtkWidget *new_window;
-		char *filename;
+	g_return_if_fail (EOG_IS_WINDOW (window));
 
+	if (window->priv->dnd_files == NULL) return;
+
+	for (l = window->priv->dnd_files; l; l = l->next) {
 		g_assert (l->data != NULL);
-		filename = l->data;
+		filename = gnome_vfs_uri_to_string (l->data, GNOME_VFS_URI_HIDE_NONE);
 
 		if (need_new_window)
 			new_window = eog_window_new ();
@@ -479,9 +499,8 @@ eog_window_drag_data_received (GtkWidget *widget,
 		}
 	}
 
-#if 0
-	gnome_uri_list_free_strings (filenames);
-#endif 
+	gnome_vfs_uri_list_free (window->priv->dnd_files);
+	window->priv->dnd_files = NULL;
 }
 
 /**
@@ -501,25 +520,13 @@ eog_window_new (void)
 	window = EOG_WINDOW (g_object_new (EOG_TYPE_WINDOW, NULL));
 
 	uic = bonobo_ui_container_new ();
-	bonobo_window_construct (BONOBO_WINDOW (window), uic, "eog", _("Eye Of Gnome"));
+	bonobo_window_construct (BONOBO_WINDOW (window), uic, "eog", _("Eye of Gnome"));
 
 	eog_window_construct (window);
 
 	return GTK_WIDGET (window);
 }
 
-#if 0
-/* Sets the sensitivity of all the items */
-static void
-sensitize_zoom_items (GtkWidget **widgets, gboolean sensitive)
-{
-	g_assert (widgets != NULL);
-
-	for (; *widgets != NULL; widgets++)
-		gtk_widget_set_sensitive (*widgets, sensitive);
-}
-#endif
-
 /* Sets the window as a drag destination */
 static void
 set_drag_dest (EogWindow *window)
@@ -538,11 +545,14 @@ set_drag_dest (EogWindow *window)
 
 static BonoboUIVerb eog_app_verbs[] = {
 	BONOBO_UI_VERB ("FileNewWindow", verb_FileNewWindow_cb),
-	BONOBO_UI_VERB ("FileOpen", verb_FileOpen_cb),
+	BONOBO_UI_VERB ("FileOpen",      verb_FileOpen_cb),
 	BONOBO_UI_VERB ("FileCloseWindow", verb_FileCloseWindow_cb),
 	BONOBO_UI_VERB ("FileExit",      verb_FileExit_cb),
-	BONOBO_UI_VERB ("Preferences", verb_Preferences_cb),
+	BONOBO_UI_VERB ("Preferences",   verb_Preferences_cb),
 	BONOBO_UI_VERB ("HelpAbout",     verb_HelpAbout_cb),
+	BONOBO_UI_VERB ("DnDNewWindow",  verb_DnDNewWindow_cb),
+	BONOBO_UI_VERB ("DnDSameWindow", verb_DnDSameWindow_cb),
+	BONOBO_UI_VERB ("DnDCancel",     verb_DnDCancel_cb),
 	BONOBO_UI_VERB_END
 };
 
@@ -557,8 +567,8 @@ void
 eog_window_construct (EogWindow *window)
 {
 	EogWindowPrivate *priv;
+	BonoboUIContainer *ui_container;
 	BonoboUIComponent *ui_comp;
-	gchar *fname;
 	GdkGeometry geometry;
 
 	g_return_if_fail (window != NULL);
@@ -566,7 +576,7 @@ eog_window_construct (EogWindow *window)
 
 	priv = window->priv;
 
-	priv->ui_container = bonobo_window_get_ui_container (BONOBO_WINDOW (window));
+	ui_container = bonobo_window_get_ui_container (BONOBO_WINDOW (window));
 
 	bonobo_ui_engine_config_set_path (
 		bonobo_window_get_ui_engine (BONOBO_WINDOW (window)),
@@ -579,16 +589,10 @@ eog_window_construct (EogWindow *window)
 	/* add menu and toolbar */
 	ui_comp = bonobo_ui_component_new ("eog");
 	bonobo_ui_component_set_container (ui_comp, 
-					   BONOBO_OBJREF (priv->ui_container), NULL);
+					   BONOBO_OBJREF (ui_container), NULL);
 
-	fname = bonobo_ui_util_get_ui_fname (NULL, "eog-shell-ui.xml");
-	if (fname && g_file_exists (fname)) {
-		bonobo_ui_util_set_ui (ui_comp, NULL, "eog-shell-ui.xml", "EOG", NULL);
-		bonobo_ui_component_add_verb_list_with_data (ui_comp, eog_app_verbs, window);
-	} else {
-		g_error (N_("Can't find eog-shell-ui.xml.\n"));
-	}
-	g_free (fname); /* we reuse this variable later! */
+	bonobo_ui_util_set_ui (ui_comp, NULL, "eog-shell-ui.xml", "EOG", NULL);
+	bonobo_ui_component_add_verb_list_with_data (ui_comp, eog_app_verbs, window);
 
 	/* add statusbar */
 	priv->statusbar = gnome_appbar_new (FALSE, TRUE, GNOME_PREFERENCES_NEVER);
@@ -597,7 +601,7 @@ eog_window_construct (EogWindow *window)
 	gtk_widget_show (GTK_WIDGET (priv->statusbar));
 
 	/* add control frame interface */
-	priv->ctrl_frame = bonobo_control_frame_new (BONOBO_OBJREF (priv->ui_container));
+	priv->ctrl_frame = bonobo_control_frame_new (BONOBO_OBJREF (ui_container));
 	bonobo_control_frame_set_autoactivate (priv->ctrl_frame, FALSE);
 	g_signal_connect (G_OBJECT (priv->ctrl_frame), "activate_uri",
 			  (GtkSignalFunc) activate_uri_cb, NULL);
@@ -605,27 +609,10 @@ eog_window_construct (EogWindow *window)
 	set_drag_dest (window);
 
 	/* set default geometry */
-	geometry.min_width = 200;
-	geometry.min_height = 200;
-	geometry.base_width = DEFAULT_WINDOW_WIDTH;
-	geometry.base_height = DEFAULT_WINDOW_HEIGHT;
-	gtk_window_set_geometry_hints(GTK_WINDOW (window), 
-				      GTK_WIDGET (window),
-				      &geometry, 
-				      GDK_HINT_BASE_SIZE | GDK_HINT_MIN_SIZE);
-	 
-#if 0
-	{
-	GtkArg args[1];
-	/* We want the window automatically shrink to the image
-	 * size. 
-	 */
-	args[0].name = "GtkWindow::auto_shrink";
-	args[0].type = GTK_TYPE_BOOL;
-	GTK_VALUE_BOOL(args[0]) = TRUE;
-	gtk_object_setv(GTK_OBJECT(window), 1, args);
-	}
-#endif
+	gtk_window_set_default_size (GTK_WINDOW (window), 
+				     DEFAULT_WINDOW_WIDTH,
+				     DEFAULT_WINDOW_HEIGHT);
+	gtk_window_set_resizable (GTK_WINDOW (window), TRUE);
 }
 
 /**
@@ -705,43 +692,6 @@ eog_window_open_dialog (EogWindow *window)
 	}
 }
 
-#if 0
-/* Picks a reasonable size for the window and zoom factor based on the image size */
-static void
-auto_size (EogWindow *window)
-{
-	EogWindowPrivate *priv;
-	GtkWidget *view;
-	Image *image;
-	int iwidth, iheight;
-	int swidth, sheight;
-	int zwidth, zheight;
-	double zoom;
-
-	priv = window->priv;
-
-	if (!window_has_image (window))
-		return;
-
-	view = ui_image_get_image_view (UI_IMAGE (priv->ui));
-	image = image_view_get_image (IMAGE_VIEW (view));
-
-	iwidth = gdk_pixbuf_get_width (image->pixbuf);
-	iheight = gdk_pixbuf_get_height (image->pixbuf);
-
-	swidth = gdk_screen_width () * 0.75;
-	sheight = gdk_screen_height () * 0.75;
-
-	zoom = zoom_fit_scale (swidth, sheight, iwidth, iheight, FALSE);
-	image_view_set_zoom (IMAGE_VIEW (view), zoom);
-
-	zwidth = floor (iwidth * zoom + 0.5);
-	zheight = floor (iheight * zoom + 0.5);
-
-	gtk_widget_set_usize (view, zwidth, zheight);
-}
-#endif
-
 static void
 property_changed_cb (BonoboListener    *listener,
 		     char              *event_name, 
@@ -766,27 +716,21 @@ static void
 check_for_control_properties (EogWindow *window)
 {
         EogWindowPrivate *priv;
-	BonoboControlFrame *ctrl_frame;
 	Bonobo_PropertyBag pb;
 	gchar *title;
 	CORBA_Environment ev;
 	gchar *mask = NULL;
 
 	priv = window->priv;
-	
-	ctrl_frame = bonobo_widget_get_control_frame (
-		BONOBO_WIDGET (priv->widget));
-	if (ctrl_frame == NULL)
-		goto on_error;
 
 	CORBA_exception_init (&ev);
 
-	pb = bonobo_control_frame_get_control_property_bag (ctrl_frame, &ev);
+	pb = bonobo_control_frame_get_control_property_bag (priv->ctrl_frame, &ev);
 	if (pb == CORBA_OBJECT_NIL)
 		goto on_error;
 
 	/* set window title */
-	title = bonobo_property_bag_client_get_value_string (pb, "window/title", &ev);
+	title = bonobo_pbclient_get_string (pb, "window/title", &ev);
 	if (title != NULL) {
 		gtk_window_set_title (GTK_WINDOW (window), title);
 		g_free (title);
@@ -797,7 +741,7 @@ check_for_control_properties (EogWindow *window)
 	}
 
 	/* set status bar text */
-	title = bonobo_property_bag_client_get_value_string (pb, "window/status", &ev);
+	title = bonobo_pbclient_get_string (pb, "window/status", &ev);
 	if (title != NULL) {
 		gchar *temp;
 		gnome_appbar_set_status (GNOME_APPBAR (priv->statusbar),
@@ -1003,39 +947,28 @@ get_collection_control_list (GList *text_uri_list)
 static void 
 add_control_to_ui (EogWindow *window, Bonobo_Control control)
 {
+	GtkWidget *widget;
 	EogWindowPrivate *priv;
 	CORBA_Environment ev;
 
 	g_return_if_fail (window != NULL);
 	g_return_if_fail (EOG_IS_WINDOW (window));
-
+	
 	priv = window->priv;
-
-	/* Remove old control if necessary */
-	if (priv->widget)
-		gtk_container_remove (GTK_CONTAINER (priv->box), priv->widget);
-
-	/* Show the new control */
-	priv->widget = bonobo_widget_new_control_from_objref
-					(control, BONOBO_OBJREF (priv->ui_container));
-	if (!priv->widget) {
-		g_warning ("Could not create a widget from the control!");
+	CORBA_exception_init (&ev);
+	
+	bonobo_control_frame_bind_to_control (priv->ctrl_frame, control, &ev);
+	widget = bonobo_control_frame_get_widget (priv->ctrl_frame);
+	
+	if (!widget) {
+	        g_warning ("Could not create a widget from the control!");
 		return;
 	}
-	gtk_container_add (GTK_CONTAINER (priv->box), priv->widget);
-	gtk_widget_show (priv->widget);
+	gtk_container_add (GTK_CONTAINER (priv->box), widget);
+	gtk_widget_show (widget);
 
-	/* Set control frame and activate control. */
-	CORBA_exception_init (&ev);
-	Bonobo_Control_setFrame (control,
-				 BONOBO_OBJREF (priv->ctrl_frame), &ev);
-	Bonobo_Control_activate (control, TRUE, &ev);
+	bonobo_control_frame_control_activate (priv->ctrl_frame);
 
-	/* Get the property control. */
-	if (priv->property_control)
-		bonobo_object_release_unref (priv->property_control, NULL);
-	priv->property_control = Bonobo_Unknown_queryInterface
-			(control, "IDL:Bonobo/PropertyControl:1.0", &ev);
 	CORBA_exception_free (&ev);
 
 	/* retrieve control properties and install listeners */
@@ -1096,6 +1029,7 @@ eog_window_open (EogWindow *window, const char *text_uri)
 
 	/* add it to the user interface */
 	add_control_to_ui (window, control);
+	bonobo_object_release_unref (control, NULL);
 
 	/* clean up */
 	gnome_vfs_file_info_unref (info);
@@ -1129,18 +1063,16 @@ eog_window_open_list (EogWindow *window, GList *text_uri_list)
 Bonobo_PropertyControl
 eog_window_get_property_control (EogWindow *window, CORBA_Environment *ev)
 {
+	Bonobo_Control control;
+	Bonobo_PropertyControl prop_control;
+
 	g_return_val_if_fail (window != NULL, CORBA_OBJECT_NIL);
 	g_return_val_if_fail (EOG_IS_WINDOW (window), CORBA_OBJECT_NIL);
 
-	return (CORBA_Object_duplicate (window->priv->property_control, ev));
-}
-
-BonoboUIContainer*
-eog_window_get_ui_container (EogWindow *window)
-{
-	g_return_val_if_fail (window != NULL, NULL);
-	g_return_val_if_fail (EOG_IS_WINDOW (window), NULL);
-	g_return_val_if_fail (window->priv->ui_container, NULL);
-
-	return window->priv->ui_container;
+	control = bonobo_control_frame_get_control (window->priv->ctrl_frame);
+	if (control == CORBA_OBJECT_NIL) return CORBA_OBJECT_NIL;
+	
+	prop_control = Bonobo_Unknown_queryInterface (control, 
+						      "IDL:Bonobo/PropertyControl:1.0", ev);
+	return prop_control;
 }
